@@ -4,8 +4,7 @@
 # imports.
 import os, sys, ast, json, glob, platform, subprocess, random
 import firebase_admin
-from firebase_admin import credentials as _credentials_
-from firebase_admin import firestore
+from firebase_admin import credentials, auth, firestore
 
 # inc imports.
 from fil3s import Files, Formats
