@@ -20,7 +20,13 @@ class Firebase(object):
 # the users class.
 class Users(object):
 	def __init__(self):
-		a=1
+
+		# the settings for sending emails.
+		self.email_address = None
+		self.email_password = None
+		self.smtp_host = "smtp.gmail.com"
+		self.smtp_port = 587
+
 		#
 	def get(self, 
 		# define one of the following parameters.
@@ -195,6 +201,124 @@ class Users(object):
 			return r3sponse.error_response(f"You are not signed in, error: {error}")
 
 		#
+	def send_code(self, 
+		# define uid or email to retrieve user.
+		uid=None, 
+		email=None, 
+		# the clients ip.
+		ip="unknown",
+		# the mode id.
+		mode="verification",
+		# the mode title.
+		html="Account Activation",
+		# the path to the html file.
+		html=None,
+	):
+
+		# check email.
+		try:
+			self.email
+		except AttributeError:
+			response = self.__initialize_email__()
+			if response["error"] != None: return response
+				
+		# save code.
+		code = Formats.Integer(0).generate(length=6)
+		response = self.get(uid=uid, email=email)
+		if response["error"] != None: return response
+		user = response["user"]
+		try: self.verification_codes[mode]
+		except: self.verification_codes[mode] = {}
+		self.verification_codes[mode][user.uid] = {
+			"code":code,
+			"stamp":Formats.Date().timestamp,
+			"attempts":3,
+		}
+
+		# send email.
+		html = utils.__load_file__(html).format(
+			code=code,
+			client=user.display_name,
+			ip=ip,
+			timestamp=Formats.Date().timestamp,
+			white=TEMPLATE_DATA["colors"]["white"],
+			dark_grey=TEMPLATE_DATA["colors"]["dark_grey"],
+			purple=TEMPLATE_DATA["colors"]["purple"],
+			grey=TEMPLATE_DATA["colors"]["grey"],
+			black=TEMPLATE_DATA["colors"]["black"],
+			tint=TEMPLATE_DATA["colors"]["backgrounds"]["top_bar"],)
+		
+		# send email.
+		response = self.email.send(
+			subject=f'{title} - Verification Code',
+			recipients=[user.email], 
+			html=html)
+		if response["error"] != None:
+			response = self.__initialize_email__()
+			if response["error"] != None: return response
+			response = self.email.send(
+				subject=f'{title} - Verification Code',
+				recipients=[user.email], 
+				html=html)
+			if response["error"] != None: return response
+
+		# success.
+		return r3sponse.success_response(f"Successfully send a verification code to user [{user.display_name}].")
+
+		#
+	def verify_code(self, 
+		# define uid or email to retrieve user.
+		uid=None, 
+		email=None, 
+		# the user entered code.
+		code=000000, 
+		# the message mode.
+		mode="verification",
+	):
+		# check mode.
+		options = list(self.verification_codes.keys())
+		if mode not in options:
+			return r3sponse.error_response(f"Selected an invalid mode [{mode}], valid options: {options}.")
+
+		# get user.
+		response = self.get(uid=uid, email=email)
+		if response["error"] != None: return response
+		user = response["user"]
+
+		# get success.
+		fail = False
+		try: self.verification_codes[mode]
+		except: self.verification_codes[mode] = {}
+		try: success = self.verification_codes[mode][user.uid]["attempts"] > 0 and str(self.verification_codes[mode][user.uid]["code"]) == str(code)
+		except KeyError: fail = True
+
+		# handle.
+		if fail: 
+			return r3sponse.error_response("Incorrect verification code.")
+		elif not success: 
+			self.verification_codes[mode][user.uid]["attempts"] -= 1
+			return r3sponse.error_response("Incorrect verification code.")
+		else:
+			del self.verification_codes[mode][user.uid]
+			return r3sponse.success_response("Successfull verification.")
+
+		#
+	# system functions.
+	def __initialize_email__(self):
+		# the email object.
+		if self.email_address == None or self.email_password == None:
+			return r3sponse.error_response("Define the firebase.users.email_address & firebase.users.email_password variables to send emails.")
+		self.email = utils.Email(
+			email=self.email_address,
+			password=self.email_password,
+			smtp_host=smpt_host,
+			smtp_port=smpt_port,)
+		response = self.email.login()
+		if response["success"]:
+			return r3sponse.success_response("Successfully initialized the mail object.")
+		else: 
+			self.email = None
+			return response
 
 # the firestore class.
 class FireStore(object):
